// Modified by Hristo Hristov on 19.08.11

// Revision 3

// Shattering Implementation from MSDN (18.09.11)

float4x4 World;
float4x4 View;
float4x4 Projection;
float specularIntensity = 0.8f;
float specularPower = 0.5f; 
float2 scaleBias;
float3 cameraPos;

float4 diffuseColor;
bool textureSet;

bool bShatter;
float time;
float3 shatterDir;
float3 rotationalVelocity;
shared static float3 oldPos = float3(1,1,1);

// ****************************************************************
// Textures
// ****************************************************************
texture Texture;
sampler diffuseSampler = sampler_state
{
    Texture = (Texture);
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

texture SpecularMap;
sampler specularSampler = sampler_state
{
    Texture = (SpecularMap);
    MagFilter = LINEAR;
    MinFilter = LINEAR;
    Mipfilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

texture NormalMap;
sampler normalSampler = sampler_state
{
    Texture = (NormalMap);
    MagFilter = LINEAR;
    MinFilter = LINEAR;
    Mipfilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

texture HeightMap;
sampler heightSampler = sampler_state
{
    Texture = (HeightMap);
    MagFilter = LINEAR;
    MinFilter = LINEAR;
    Mipfilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

// ****************************************************************
// Structs
// ****************************************************************
struct VertexShaderInput
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
    float2 TexCoord : TEXCOORD0;
    float3 Binormal : BINORMAL0;
    float3 Tangent : TANGENT0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float2 Depth : TEXCOORD1;
    float3 ViewDir : TEXCOORD2;
	float3x3 tangentToWorld : TEXCOORD3;	
};

struct PixelShaderOutput
{
    half4 Color : COLOR0;
    half4 Normal : COLOR1;
    half4 Depth : COLOR2;
};


// ****************************************************************
// Helper Methods
// ****************************************************************
// Helper function to create a YawPitchRoll Matrix. 
// Used to rotate the verticies by the random rotational values generated in the 
// processor.
float4x4 CreateYawPitchRollMatrix(float x, float y, float z)
{
    float4x4 result;
        
    result[0][0] = cos(z)*cos(y) + sin(z)*sin(x)*sin(y);
    result[0][1] = -sin(z)*cos(y) + cos(z)*sin(x)*sin(y);
    result[0][2] = cos(x)*sin(y);
    result[0][3] = 0;
    
    result[1][0] = sin(z)*cos(x);
    result[1][1] = cos(z)*cos(x);
    result[1][2] = -sin(x);
    result[1][3] = 0;
    
    result[2][0] = cos(z)*-sin(y) + sin(z)*sin(x)*cos(y);
    result[2][1] = sin(z)*sin(y) + cos(z)*sin(x)*cos(y);
    result[2][2] = cos(x)*cos(y);
    result[2][3] = 0;
    
    result[3][0] = 0;
    result[3][1] = 0;
    result[3][2] = 0;
    result[3][3] = 1;    

    return result;
}


// ****************************************************************
// Vertex Shader
// ****************************************************************
VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

	float3 position;

	//// If enabled, shatter the model
	//if(bShatter)
	//{
		//float4x4 rotMatrix = CreateYawPitchRollMatrix(rotationalVelocity.x, 
                                                      //rotationalVelocity.y, 
                                                      //rotationalVelocity.z );
		//
		//float3 center = input.Position.xyz - oldPos;//float3(input.Normal.y, input.Normal.x, input.Normal.z);
		//// Rotate the vertex around its triangle's center
		//position = input.Position.xyz;//position = input.Position + mul(input.Position - center, rotMatrix);//input.Position + mul(input.Position - input.Position / 3, rotMatrix);
    //
		//// Displace the vertex along its normal
		//position += mul(input.Normal * oldPos, rotMatrix) * 5 * time;    
    //
		//// Move the vertex downward as a function of time^2 to give a nice curvy falling
		//// effect.
		////position.y -= time*time * 200; 		
	//}
	//else
		position = input.Position.xyz;

		//oldPos = oldPos * -1;

    float4 worldPosition = mul(float4(position/*input.Position.xyz*/,1), World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	
    output.TexCoord = input.TexCoord;

    output.Depth.x = output.Position.z;
    output.Depth.y = output.Position.w;
	//output.Depth.z = viewPosition.z;

    // calculate tangent space to world space matrix using the world space tangent,
    // binormal, and normal as basis vectors
    output.tangentToWorld[0] = mul(input.Tangent, World);
    output.tangentToWorld[1] = mul(input.Binormal, World);
    output.tangentToWorld[2] = mul(input.Normal, World);

	float3 viewDir = cameraPos - worldPosition.xyz;

	float3x3 tbnMatrix = float3x3(input.Tangent.x, input.Binormal.x, input.Normal.x,
	                              input.Tangent.y, input.Binormal.y, input.Normal.y,
	                              input.Tangent.z, input.Binormal.z, input.Normal.z);

	output.ViewDir = mul(viewDir, tbnMatrix);

    return output;
}


// ****************************************************************
// Pixel Shader
// ****************************************************************
PixelShaderOutput PixelShaderFunction(VertexShaderOutput input)
{
    PixelShaderOutput output;

	float3 v = normalize(input.ViewDir);

	float height = tex2D(heightSampler, input.TexCoord).r;
        
	height = height * scaleBias.x + scaleBias.y;
	float2 texCoord = input.TexCoord + (height * v.xy);
		
	if(textureSet)
		output.Color = tex2D(diffuseSampler, texCoord);
	else
		output.Color = diffuseColor;
    
    float4 specularAttributes = tex2D(specularSampler, texCoord);
    //specular Intensity
    output.Color.a = specularAttributes.r;
    
    // read the normal from the normal map
    float3 normalFromMap = tex2D(normalSampler, texCoord);
    //tranform to [-1,1]
    normalFromMap = 2.0f * normalFromMap - 1.0f;
    //transform into world space
    normalFromMap = mul(normalFromMap, input.tangentToWorld);
    //normalize the result
    normalFromMap = normalize(normalFromMap);
    //output the normal, in [0,1] space
    output.Normal.rgb = 0.5f * (normalFromMap + 1.0f);

    //specular Power
    output.Normal.a = specularAttributes.a;

	// Screen space
    output.Depth = input.Depth.x / input.Depth.y;

	// View space
	//output.Depth.g = input.Depth.z;

    return output;
}


// ****************************************************************
// Techniques
// ****************************************************************
technique Technique1
{
    pass Pass1
    {
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}
